/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#ifndef __NetGroup__
#define __NetGroup__

/** The network class for global settings like timeouts.
    All the timeouts are in SECONDS.*/
class NetGroup
{
	public:
		NetGroup();
		NetGroup(const char *);
		
		/** Get the current connection timeout.*/
		int getConnectionTimeout();
		/** Get the current repling timeout.*/
		int getReplyTimeout();
		/** Get the current reading timeout.*/
		int getReadTimeout();
		/** Get the current writing timeout.*/
		int getWriteTimeout();
		
		/** Get the default value for connecting operation.*/
		static int getDefaultConnectionTimeout();
		/** Get the default value for repling operation.*/
		static int getDefaultReplyTimeout();
		/** Get the default value for reading operation.*/
		static int getDefaultReadTimeout();
		/** Get the default value for writing operation.*/
		static int getDefaultWriteTimeout();
	private:
	  /** connection timeout*/
		int connectionTimeout;
		/** repling timeout*/
		int replyTimeout;
		/** reading timeout*/
		int readTimeout;
		/** writing timeout*/
		int writeTimeout;
};

#endif
