/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#ifndef __ServerCommGate__
#define __ServerCommGate__

/*
This class defines the "server communication gate" object.
It is part of the skeleton server code.
The object accepts incoming client connections, authenticates them if necessary and provides
services and additionally replies to all the requests a client might have. 
In order to achieve this the server communication gate uses the server object request broker.
It may be used by other server components when the server needs to act upon the client.
At the time being only the "thisInstance" loopback interface, non-multithreading, therefore
call-based only is implemented.
*/

#include <qstring.h>

class CommFrame;

class ServerCommGate
{
	public:
		ServerCommGate();
		~ServerCommGate();
		static ServerCommGate *thisInstance();
		
		void callReceive(CommFrame &inFrame, CommFrame &outFrame);
		void callReceive(CommFrame &inFrame);
		
		void sendMessage(CommFrame &inFrame);
		void sendMessage(CommFrame &inFrame, CommFrame &outFrame);
		
		QString lastError();
		bool isLocked();
	private:
		void callSend(CommFrame &inFrame);
		void callSend(CommFrame &inFrame, CommFrame &outFrame);
		
		QString err;
		bool lock;
};

#endif

