/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <imapmailframe.h>
#include <videotoolbox.h>
#include <names.h>
#include <definedmessages.h>
#include <qdir.h>
#include <qfiledialog.h>

extern VideoToolBox vtb;

IMAPMailFrame::IMAPMailFrame(QWidget *parent, QString name,
	int x, int y, int dx, int dy, int idx) :
	QFrame(parent, name)
{
	index=idx;
	setGeometry(x, y, dx, dy);
	setFrameStyle(0);
	// Creating the continue-cancel buttons
	// The buttons are going to be in the bottom right
	int x,y;
	x=dx-280, y=dy-vtb.fontHeight();
	backButton=vtb.setUpQPushButton(this,"Back",
		80, vtb.fontHeight(), &x, &y);
	x=dx-180, y=dy-vtb.fontHeight();
	continueButton=vtb.setUpQPushButton(this,"Next",
		80, vtb.fontHeight(), &x, &y);
	continueButton->setDefault(true);
	x=dx-80, y=dy-vtb.fontHeight();
	finishButton=vtb.setUpQPushButton(this,"Cancel",
		80, vtb.fontHeight(), &x, &y);
	// create the text
	int DX=5, DY=5;
	vtb.setUpFonts(QFont("helvetica",12,QFont::Normal));
	text=vtb.setUpLabelSize(this,"This will create an IMAP account.",
		dx-vtb.fontHeight()-10, &DX, &DY);
	DY+=vtb.fontHeight()+vtb.offsetY();
	DX=5;
	QString m("Mailbox name:");
	QString m1("Password:");
	int align=vtb.stringLength(m);
	serverLabel=vtb.setUpLabelSize(this,"Server name:",align, &DX, &DY);
	serverEdit=vtb.setUpLineEdit(this, 120, vtb.fontHeight(),
		&DX, &DY);
	
	portLabel=vtb.setUpLabelSize(this,"Port:", vtb.stringLength(m1), &DX, &DY);
	portEdit=vtb.setUpLineEdit(this, vtb.stringLength("MMM"), vtb.fontHeight(),
		&DX, &DY);
	DY+=vtb.fontHeight()+vtb.offsetY();
	DX=5;
	userNameLabel=vtb.setUpLabelSize(this,"User name:",align, &DX, &DY);
	userNameEdit=vtb.setUpLineEdit(this, 120, vtb.fontHeight(), &DX, &DY);
	passwordLabel=vtb.setUpLabel(this,"Password:", &DX, &DY);
	passwordEdit=vtb.setUpLineEdit(this, 120, vtb.fontHeight(), &DX, &DY);
	passwordEdit->setEchoMode(QLineEdit::Password);
	DY+=vtb.fontHeight()+vtb.offsetY();
	DX=5;
	rememberPassBox=vtb.setUpQCheckBox(this,"Remember password.",
		&DX, &DY);	
	DY+=vtb.fontHeight()+vtb.offsetY();
	DX=5;
//	mailboxLabel=vtb.setUpLabelSize(this,"Mailbox name:",align, &DX, &DY);
//	mailboxEdit=vtb.setUpLineEdit(this, 120, vtb.fontHeight(), &DX, &DY);

	connect(backButton, SIGNAL(clicked()), this, SLOT(backClicked()));
	connect(continueButton, SIGNAL(clicked()), this, SLOT(continueClicked()));
	connect(finishButton, SIGNAL(clicked()), this, SLOT(finishClicked()));
	connect(rememberPassBox, SIGNAL(clicked()), this, SLOT(passClicked()));
	init();
}
void IMAPMailFrame::continueClicked()
{
	QStringList rez;
	rez.append("[IMAP]");
	rez.append(QString("Server=")+serverEdit->text());
	rez.append(QString("Port=")+portEdit->text());
	rez.append(QString("RememberPassword=")	+
		QString((rememberPassBox->isChecked())?"Yes":"No"));
	rez.append(QString("UserName=")+userNameEdit->text());
	if(rememberPassBox->isChecked())
		rez.append(QString("Password=")+passwordEdit->text());
//	rez.append(QString("Mailbox=")+mailboxEdit->text());
	emit result(ContinueMessage, index, rez);
	close();
}

void IMAPMailFrame::finishClicked()
{
	emit result(CancelMessage, index, QStringList(""));
	close();
}

void IMAPMailFrame::backClicked()
{
	emit result(BackMessage, index, QStringList(""));
	close();
}

void IMAPMailFrame::init()
{
	portEdit->setText("143");
	serverEdit->setText(Names::systemHostDomainName());
	userNameEdit->setText(Names::systemLoginName());
//	mailboxEdit->setText("Inbox");
	passwordEdit->setEnabled(false);
	passwordLabel->setEnabled(false);
}
void IMAPMailFrame::passClicked()
{
	bool b=rememberPassBox->isChecked();
	passwordEdit->setEnabled(b);
	passwordLabel->setEnabled(b);
}





