/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#include <stdio.h>
#include <simpleconfig.h>
#include <qdict.h>
#include <qstring.h>
#include <qstringlist.h>

SimpleConfig::SimpleConfig(QStringList cfg)
{
	QDict<QString> *cg;
	for(int i=0;i<cfg.count();i++)
	{
		if(!cfg[i].isEmpty())
			if(cfg[i][0]=='[')
			{
				// create a new group
				cg=new QDict<QString>;
				currentGroup=cfg[i].mid(1, cfg[i].length()-2);
				values.insert(currentGroup, cg);

				// debug
				printf("New group: %s\n", (const char *)currentGroup);
			}
			else
			{
				// add another value to the current group
				cg->insert(cfg[i].left(cfg[i].find('=')), new QString(cfg[i].mid(cfg[i].find('=')+1)));

				printf("Adding value (%s,%s)\n", (const char *)cfg[i].left(cfg[i].find('=')), (const char *)cfg[i].mid(cfg[i].find('=')+1));
			}
	}
	
	currentGroup=QString::null;
}

SimpleConfig::~SimpleConfig()
{
	QDictIterator<QDict<QString> > it(values);
	while(it.current())
	{
		delete(it.current());
		++it;
	}
}

void SimpleConfig::setGroup(QString group)
{
	if(values[group])
		currentGroup=group;
}

QString SimpleConfig::readEntry(QString key)
{
	if(currentGroup.isNull())
		return QString::null;
	if((*values[currentGroup])[key])
		return QString(*(*values[currentGroup])[key]);
	return QString::null;
}
