/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
 
#ifndef __FolderTree__
#define __FolderTree__

#include <qlistview.h>
#include <foldertreeitem.h>

#define FOLDERTREE_MOVE 1
#define FOLDERTREE_RENAME 2
#define FOLDERTREE_MARKALLREAD 3
#define FOLDERTREE_DELETE 4

class QPopupMenu;

class FolderTree : public QListView
{
	Q_OBJECT
	public:
		FolderTree(QWidget *parent=0, const char *name=0);
		void integrateRootDirectory();
		void integrateHomeDirectory();
		void initFolderManagement();
	public slots:
		void deleteCurrentFolder();
		void renameCurrentFolder();
		void moveCurrentFolder();
		void createFolder();
		void editProperties();
		void openInNewWindow();
		void markAllRead();
	signals:
		void currentChanged(FolderTreeItem *);
	private slots:
		void ackCurrentChanged(QListViewItem *);
		void showPopup(QListViewItem *, const QPoint &, int);
		void expandItem(QListViewItem *);
		void collapseItem(QListViewItem *);
	protected:
		virtual void contentsDragEnterEvent(QDragEnterEvent *);
		virtual void contentsDragLeaveEvent(QDragLeaveEvent *);
		virtual void contentsDragMoveEvent(QDragMoveEvent *);
		virtual void contentsDropEvent(QDropEvent *);
	private:
		
		bool actionsAllowed(const QString &);
		
		QPopupMenu *folderMenu;
		QListViewItem *m_dndSourceFolder;
};

#endif
