/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#ifndef HAS_RRULE
	#define FAS_RRULE

#include <qvaluelist.h>
#include <qstringlist.h>
#include <qdatetime.h>

struct Occurences
{
	QValueList<QDate> dates;
	QValueList<QTime> times;
};

struct ByDay
{
	QString literal;
	int value;
};

class RRule
{
	static QStringList weekdayAbbreviations;
	static bool abbreviationsInitialized;
	QString frequency;
	QDateTime endDate;
	bool endDateValid;
	QDateTime referenceDate;
	int count;
	int interval;
	QValueList<int> 	bySecondList;
	QValueList<int> 	byMinuteList;
	QValueList<int> 	byHourList;
	QValueList<ByDay> byDayList;
	QValueList<int> 	byMonthDayList;
	QValueList<int> 	byYearDayList;
	QValueList<int> 	byWeekNoList;
	QValueList<int> 	byMonthList;
	QValueList<int> 	bySetPosList;
	QStringList     	byWeekDayList;
	
	QString weekDayToString(int weekday);
	int weekDayToInt(QString weekday);
	void initWeekdayString();
	void parseString(QString str);
	QValueList<int> parseStringInt(QString str);
	QValueList<ByDay> parseStringByDay(QString str);
	QStringList parseStringString(QString str);
	void matchList(QString title, QString list);
public:
	RRule(QString rrule);
	Occurences getAllOccurences(QDateTime dt, bool dateOnly);
	QValueList<QDateTime> getOccurences(QDateTime dt, bool dateOnly);
	QValueList<QDateTime> getGlobalOccurences(QDateTime reference, 
			QDateTime dt,
			bool dateOnly);
	void setReferenceDate(QDateTime d) { referenceDate=d; };
};


#endif








